// Copyright (C) 2005 Christian Mendl
//
// a simple 2D index integer structure

#ifndef INDEX2_H
#define INDEX2_H


struct Index2
{
	int x, y;

	Index2();
	Index2(int x, int y);
};


inline Index2::Index2() {
}

inline Index2::Index2(int x, int y)
{
	this->x = x;
	this->y = y;
}


inline Index2 operator+ (const Index2& v0, const Index2& v1)
{
	return Index2(v0.x + v1.x, v0.y + v1.y);
}


inline Index2 operator- (const Index2& v0, const Index2& v1)
{
	return Index2(v0.x - v1.x, v0.y - v1.y);
}


inline Index2 operator- (const Index2& v)
{
	return Index2(-v.x, -v.y);
}


inline Index2 operator* (const Index2& v, const int t)
{
	return Index2(v.x*t, v.y*t);
}


inline Index2 operator* (const int t, const Index2& v)
{
	return Index2(v.x*t, v.y*t);
}


// dot product
inline int operator* (const Index2& v0, const Index2& v1)
{
	return v0.x*v1.x + v0.y*v1.y;
}


inline void operator+= (Index2& v, const Index2& v1)
{
	v.x += v1.x;
	v.y += v1.y;
}


inline void operator-= (Index2& v, const Index2& v1)
{
	v.x -= v1.x;
	v.y -= v1.y;
}


inline void operator*= (Index2& v, const int t)
{
	v.x *= t;
	v.y *= t;
}


inline bool operator== (const Index2& l, const Index2& r)
{
	return l.x == r.x && l.y == r.y;
}


inline bool operator< (const Index2& l, const Index2& r)
{
	if (l.x < r.x)
		return true;
	else if (r.x < l.x)
		return false;

	return l.y < r.y;
}



#endif
