// Copyright (C) 2005 Christian Mendl
//
// a simple 3D index integer structure

#ifndef INDEX3_H
#define INDEX3_H


struct Index3
{
	int x, y, z;

	Index3();
	Index3(int x, int y, int z);
};


inline Index3::Index3() {
}

inline Index3::Index3(int x, int y, int z)
{
	this->x = x;
	this->y = y;
	this->z = z;
}


inline Index3 operator+ (const Index3& v0, const Index3& v1)
{
	return Index3(v0.x + v1.x, v0.y + v1.y, v0.z + v1.z);
}


inline Index3 operator- (const Index3& v0, const Index3& v1)
{
	return Index3(v0.x - v1.x, v0.y - v1.y, v0.z - v1.z);
}


inline Index3 operator- (const Index3& v)
{
	return Index3(-v.x, -v.y, -v.z);
}


inline Index3 operator* (const Index3& v, const int t)
{
	return Index3(v.x*t, v.y*t, v.z*t);
}


inline Index3 operator* (const int t, const Index3& v)
{
	return Index3(v.x*t, v.y*t, v.z*t);
}


// dot product
inline int operator* (const Index3& v0, const Index3& v1)
{
	return v0.x*v1.x + v0.y*v1.y + v0.z*v1.z;
}


// cross product
inline Index3 operator% (const Index3& v0, const Index3& v1)
{
	return Index3(
		v0.y*v1.z - v0.z*v1.y,
		v0.z*v1.x - v0.x*v1.z,
		v0.x*v1.y - v0.y*v1.x);
}


inline void operator+= (Index3& v, const Index3& v1)
{
	v.x += v1.x;
	v.y += v1.y;
	v.z += v1.z;
}


inline void operator-= (Index3& v, const Index3& v1)
{
	v.x -= v1.x;
	v.y -= v1.y;
	v.z -= v1.z;
}


inline void operator*= (Index3& v, const int t)
{
	v.x *= t;
	v.y *= t;
	v.z *= t;
}


inline int LengthSquare(const Index3& v)
{
	return v*v;
}


inline bool operator== (const Index3& l, const Index3& r)
{
	return l.x == r.x && l.y == r.y && l.z == r.z;
}


inline bool operator< (const Index3& l, const Index3& r)
{
	if (l.x < r.x)
		return true;
	else if (r.x < l.x)
		return false;

	if (l.y < r.y)
		return true;
	else if (r.y < l.y)
		return false;

	return l.z < r.z;
}



#endif