// Copyright (C) 2005 Christian Mendl
//
// character strings

#ifndef STR_H
#define STR_H

#include <string.h>
#include "array.h"


class Str
{

public:

	Str();
	explicit Str(int length);	// length without terminating character '\0'
	Str(const char *s);

	Str(const Str& that);

	Str& operator+= (const char *right);
	Str& operator+= (const Str& right);

	friend Str operator+ (const Str& left, const Str& right);

	operator const char * () const;

	char *CStr();
	const char *CStr() const;

	// without terminating character '\0'
	int Length() const;

	void StripBack(int num);	// strips 'num' characters from back

	static char *VarArg(const char *s, ...);

private:

	// stores terminating character '\0'
	Array<char> data;
};


inline Str::Str()
: data(1)
{
	data[0] = '\0';
}

inline Str::Str(int length)
: data(length+1)
{
	data.Back() = '\0';
}

inline Str::Str(const char *s)
: data(int(strlen(s)+1))
{
	memcpy(data.Data(), s, data.NumBytes());
}

inline Str::Str(const Str& that)
: data(that.data) {
}


inline Str::operator const char * () const
{
	return data.Data();
}


inline char *Str::CStr()
{
	return data.Data();
}

inline const char *Str::CStr() const
{
	return data.Data();
}


inline int Str::Length() const
{
	return data.Size() - 1;
}


inline void Str::StripBack(int num)
{
	assert(num < data.Size());	// data contains '\0' character

	data.Shrink(num);
	data.Back() = '\0';
}



#endif
