// Copyright (C) 2005 Christian Mendl

#include "array.h"
#include <iostream>
using namespace std;


template<class T>
void Print(const Array<T>& a)
{
	cout << "Size: " << a.Size() << "; Number of bytes: " << a.NumBytes() << endl;
	if (!a.Empty())
	{
		cout << "Elements: ";
		for (int i = 0; i < a.Size()-1; i++)
			cout << a[i] << ", ";
		cout << a.Back() << endl;
	}
}

template<class T>
void Search(const Array<T>& a, const T& t)
{
	cout << endl << "Looking for " << t << "..." << endl;
	int i = a.Find(t);
	if (i == -1)
		cout << "not found." << endl;
	else
		cout << t << " is at index " << i << "." << endl;
}


void main()
{
	cout << "Creating 'int' array..." << endl;
	Array<int> a;
	if (a.Empty())
		cout << "Array is empty." << endl;
	else
		cout << "Array is not empty." << endl;

	cout << endl << "Pushing back 2 elements..." << endl;
	a.PushBack(4);
	a.PushBack(6);
	Print(a);

	cout << endl << "Assigning new entry at second index..." << endl;
	a[1] = -1;
	cout << "a[1] == " << a[1] << endl;

	cout << endl << "Resizing..." << endl;
	a.Resize(5);
	a[2] = 17;
	a[3] = 18;
	a.Back() = -5;
	Print(a);

	Search(a, 0);
	Search(a, -5);

	cout << endl << "Popping last entry..." << endl;
	a.PopBack();
	Print(a);

	cout << endl << "Erasing element at index 1..." << endl;
	a.EraseAt(1);
	Print(a);

	cout << endl << "Erasing '4'..." << endl;
	if (a.Erase(4))
		cout << "successfully erased." << endl;
	else
		cout << "erasing failed." << endl;
	Print(a);

	cout << endl << "Creating 'double' array..." << endl;
	Array<double> a2(6);
	a2[5] = 3.84593;
	Print(a2);

	cout << endl << "Appending first array to second..." << endl;
	a2.PushBack(a);
	Print(a2);

	cout << endl << "Clearing first array..." << endl;
	a.Clear();
	Print(a);
}
