// Copyright (C) 2005 Christian Mendl

#include "list.h"
#include <iostream>

using namespace std;


template<class T>
void Print(const List<T>& l)
{
	List<T>::ConstIterator it;

	cout << "Size: " << l.Size() << endl;

	cout << "Elements: ";
	for (it = l.Begin(); it != l.End(); ++it)
		cout << *it << " ";

	cout << endl << "Reverse: ";
	for (it = l.ReverseBegin(); it != l.ReverseEnd(); --it)
		cout << *it << " ";

	cout << endl;
}

template<class T>
void Search(const List<T>& l, const T& t)
{
	cout << "Looking for " << t << "..." << endl;
	List<T>::ConstIterator it = l.Find(t);
	if (it != l.End())
		cout << t << " is in list." << endl;
	else
		cout << "not found." << endl;
}


void main()
{
	cout << "Creating double-linked list..." << endl;
	List<int> l;

	cout << endl << "Appending some integers..." << endl;
	l.PushBack(3);
	l.PushBack(7);
	l.PushBack(0);
	Print(l);

	cout << endl << "Inserting elements at the beginning..." << endl;
	l.PushFront(-1);
	l.PushFront(13);
	Print(l);

	Search(l, 2);
	Search(l, 3);

	cout << endl << "Changing front and back..." << endl;
	l.Front() = -2;
	l.Back() = 8;
	Print(l);

	cout << endl << "Erasing 2 and 3..." << endl;
	l.Erase(2);
	l.Erase(3);
	Print(l);

	cout << endl << "Erasing front element" << endl;
	l.Erase(l.Begin());
	Print(l);

	cout << endl << "Inserting 5 before last element..." << endl;
	List<int>::Iterator it = l.ReverseBegin();
	l.Insert(it, 5);
	Print(l);

	cout << endl << "Popping front and back elements..." << endl;
	l.PopFront();
	l.PopBack();
	Print(l);

	cout << endl << "Clearing..." << endl;
	l.Clear();
	Print(l);
}
