// Copyright (C) 2005 Christian Mendl

#include "map.h"
#include <iostream>
using namespace std;


template<class K, class V>
void Print(const Map<K, V>& m)
{
	Map<K, V>::Iterator it;

	cout << "Size: " << m.Size() << endl;

	cout << "Elements: ";
	for (it = m.Begin(); it != m.End(); ++it)
		cout << "(" << it.Key() << ", " << *it << ") ";

	cout << endl << "Reverse: ";
	for (it = m.ReverseBegin(); it != m.End(); --it)
		cout << "(" << it.Key() << ", " << *it << ") ";

	cout << endl;
}

template<class K, class V>
void Search(const Map<K, V>& m, const K& k)
{
	cout << endl << "Looking for " << k << "..." << endl;
	Map<K, V>::Iterator it = m.Find(k);
	if (it != m.End())
	{
		assert(m.Contains(k));
		assert(it.Key() == k);
		cout << "(" << k << ", " << *it << ") is in set." << endl;
	}
	else
	{
		assert(!m.Contains(k));
		cout << "not found." << endl;
	}
}


struct T
{
	T() {
	}

	T(int _i, char _c)
	: i(_i), c(_c) {
	}

	int i;
	char c;
};

inline bool operator< (const T& left, const T& right)
{
	if (left.i < right.i)
		return true;
	else if (right.i < left.i)
		return false;

	return left.c < right.c;
}

ostream& operator<< (ostream& strm, const T& t)
{
	strm << "(" << t.i << ", " << t.c << ")";
	return strm;
}


void main()
{
	cout << "Creating new <int, char> map..." << endl;
	Map<int, char> m;

	cout << endl << "Inserting (10, 'h'), (5, 'i') and (1, 'a')..." << endl;
	m.InsertCertain(10, 'h');
	m.InsertCertain(5, 'i');
	m.InsertCertain(1, 'a');
	Print(m);

	cout << endl << "Trying to insert (5, 'k')..." << endl;
	if (m.Insert(5, 'k') == m.End())
		cout << "(5, 'k') inserted." << endl;
	else
		cout << "key 5 already in tree." << endl;

	Search(m, 1);
	Search(m, 2);

	cout << endl << "Erasing key 5..." << endl;
	m.Erase(5);
	Print(m);

	cout << endl << "Assigning 'f' to key 10..." << endl;
	m[10] = 'f';
	Print(m);

	cout << endl << "Assigning 'z' to key -1..." << endl;
	m[-1] = 'z';
	Print(m);

	cout << endl << "Creating new map by copy constructor..." << endl;
	Map<int, char> m2(m);
	Print(m2);

	cout << endl << "Clearing..." << endl;
	m2.Clear();
	Print(m2);

	cout << endl << "Creating another map with different key and data; only operator< must be defined";
	Map<T, double> m3;
	m3.Insert(T(6, 'm'), 1.23);
	m3.Insert(T(5, 'g'), 2.34);
	m3[T(5, 'f')] = 3.45;
	Print(m3);
}
