// Copyright (C) 2005 Christian Mendl

#include "set.h"
#include <iostream>
using namespace std;


template<class K>
void Print(const Set<K>& s)
{
	Set<K>::Iterator it;

	cout << "Size: " << s.Size() << endl;

	cout << "Elements: ";
	for (it = s.Begin(); it != s.End(); ++it)
		cout << *it << " ";

	cout << endl << "Reverse: ";
	for (it = s.ReverseBegin(); it != s.End(); --it)
		cout << *it << " ";

	cout << endl;
}

template<class K>
void Search(const Set<K>& s, const K& k)
{
	cout << endl << "Looking for " << k << "..." << endl;
	if (s.Contains(k))
	{
		assert(s.Find(k) != s.End());
		cout << k << " is in set." << endl;
	}
	else
	{
		assert(s.Find(k) == s.End());
		cout << "not found." << endl;
	}
}


void main()
{
	cout << "Creating 'int' set" << endl;
	Set<int> s;

	cout << endl << "Inserting some integers..." << endl;
	s.InsertCertain(7);
	s.InsertCertain(-1);
	s.Insert(5);
	s.Insert(8);
	s.Insert(7);
	s.Insert(13);
	s.Insert(0);
	Print(s);

	Search(s, 2);
	Search(s, 5);

	cout << endl << "Trying to erase 5 and 6..." << endl;
	s.Erase(5);
	s.Erase(6);
	Print(s);

	cout << endl << "Creating new set by copy constructor..." << endl;
	Set<int> s2(s);
	Print(s2);

	cout << endl << "Erasing first integer..." << endl;
	s2.Erase(s2.Begin());
	Print(s2);

	cout << endl << "Assigning first to second..." << endl;
	s2 = s;
	Print(s2);

	cout << endl << "Clearing..." << endl;
	s.Clear();
	Print(s);
}
