// Copyright (C) 2005 Christian Mendl

#include "unique.h"
#include <iostream>
using namespace std;


template<class T>
void Print(const Unique<T>& u)
{
	cout << "Size: " << u.Size() << "; Number of bytes: " << u.NumBytes() << endl;
	if (!u.Empty())
	{
		cout << "Elements: ";
		for (int i = 0; i < u.Size()-1; i++)
			cout << u[i] << ", ";
		cout << u.Back() << endl;
	}
}

template<class T>
void Search(const Unique<T>& u, const T& t)
{
	cout << endl << "Looking for " << t << "..." << endl;
	int i = u.Find(t);
	if (i == -1)
		cout << "not found." << endl;
	else
		cout << t << " is at index " << i << "." << endl;
}


void main()
{
	cout << "Creating 'int' unique array..." << endl;
	Unique<int> u;
	Print(u);

	cout << endl << "Pushing back '4, 2, -1, 2, 3'..." << endl;
	u.PushBack(4);
	u.PushBack(2);
	u.PushBack(-1);
	u.PushBack(2);
	u.PushBack(3);
	Print(u);

	Search(u, 0);
	Search(u, 2);

	cout << endl << "Erasing element at index 1..." << endl;
	u.EraseAt(1);
	Print(u);

	cout << endl << "Erasing element '4'..." << endl;
	if (u.Erase(4))
		cout << "successfully erased." << endl;
	else
		cout << "erasing failed." << endl;
	Print(u);

	cout << endl << "Creating another unique array and adding some elements..." << endl;
	Unique<int> u2;
	u2.PushBack(4);
	u2.PushBack(3);
	u2.PushBack(5);
	Print(u2);

	cout << endl << "Appending array to first..." << endl;
	u.PushBack(u2);
	Print(u);

	cout << endl << "Clearing..." << endl;
	u.Clear();
	Print(u);

	cout << endl << "Reinserting some elements..." << endl;
	u.PushBack(0);
	u.PushBack(1);
	Print(u);
}
