// Copyright (C) 2005 Christian Mendl

#include "unique.h"
#include "set.h"
#include <time.h>
#include <iostream>
using namespace std;


template<class T>
void Print(const Unique<T>& u)
{
	cout << "Size: " << u.Size() << "; Number of bytes: " << u.NumBytes() << endl;
	if (!u.Empty())
	{
		cout << "Elements: ";
		for (int i = 0; i < u.Size()-1; i++)
			cout << u[i] << ", ";
		cout << u.Back() << endl;
	}
}


template<class T>
void Check(Unique<T>& u)
{
	assert(u.Size() > 1);

	for (int i = 0; i < u.Size(); i++)
		if (u.Find(u[i]) != i)
			cerr << "error in 'find'";

	Set<T> set;
	for (int i = 0; i < u.Size(); i++)
		set.Insert(u[i]);

	if (u.Size() != set.Size())
		cerr << "doublicated element" << endl;

	Unique<T> u2(u);

	Unique<T> u3;
	if (!u3.Empty())
		cerr << "new array not empty" << endl;

	u3.PushBack(u.Front());

	for (int i = 1; i < u.Size(); i++)
		if (u3.Find(u[i]) != -1)
			cerr << "error in 'find'";

	if (u3.Size() != 1)
		cerr << "wrong size" << endl;

	u3.EraseAt(0);

	if (u3.Size() != 0)
		cerr << "array not empty" << endl;

	u3 = u2;

	if (u.Size() != u2.Size())
		cerr << "sizes don't match" << endl;

	if (u2.Size() != u3.Size())
		cerr << "sizes don't match" << endl;

	u3.Clear();

	if (!u3.Empty())
		cerr << "array not empty" << endl;

	u3 = u2;

	if (u2.Size() != u3.Size())
		cerr << "sizes don't match" << endl;

	if (u2.PushBack(u[0]) != 0)
		cerr << "error inserting element" << endl;

	if (u3.PushBack(u[0]) != 0)
		cerr << "error inserting element" << endl;

	if (u.Size() != u2.Size())
		cerr << "sizes don't match" << endl;

	for (int i = 0; i < u.Size(); i++)
	{
		if (u[i] != u2[i])
			cerr << "different elements" << endl;

		if (u[i] != u3[i])
			cerr << "different elements" << endl;
	}

	u.PushBack(u2);
	if (u.Size() != u2.Size())
		cerr << "error inserting array" << endl;

	for (int i = 0; i < u.Size(); i++)
		if (u[i] != u2[i])
			cerr << "different elements" << endl;

	T t = u.Back();
	u.PopBack();

	if (u.Size() != u2.Size() - 1)
		cerr << "wrong size" << endl;

	u.PushBack(t);

	if (u.Size() != u2.Size())
		cerr << "wrong size" << endl;

	if (u.Back() != u2.Back())
		cerr << "error inserting element" << endl;

	for (int i = 0; i < u.Size(); i++)
		if (u2.Find(u[i]) != i)
			cerr << "error in 'find'";

	set.Clear();

	while (u.Size() > u2.Size()/2)
	{
		set.InsertCertain(u[1]);
		u.EraseAt(1);
	}

	if (set.Size() != (u2.Size()+1)/2)
		cerr << "wrong size" << endl;

	for (Set<T>::Iterator it = set.ReverseBegin(); it != set.End(); --it)
	{
		int sz = u.Size();
		if (u.PushBack(*it) != sz)
			cerr << "error re-inserting element" << endl;
	}

	set.Clear();

	while (!u.Empty())
	{
		t = u[0];
		if (!u.Erase(t))
			cerr << "error in erase" << endl;
		set.InsertCertain(t);
	}

	u2.Shrink(u2.Size()/2);

	if (u2.Size() != (u3.Size()+1)/2)
		cerr << "wrong size" << endl;

	for (int i = 0; i < u2.Size(); i++)
		if (u2[i] != u3[i])
			cerr << "wrong elements after shrink" << endl;

	for (int i = u3.Size()-1; i >= (u3.Size()+1)/2; i--)
	{
		int sz = u2.Size();
		if (u2.PushBack(u3[i]) != sz)
			cerr << "error re-inserting element" << endl;
	}

	for (int i = 0; i < (u3.Size()+1)/2; i++)
		if (u2.PushBack(u3[i]) != i)
			cerr << "error inserting existing element" << endl;
}


void main()
{
	Unique<int> u;

	u.PushBack(4);
	u.PushBack(2);
	u.PushBack(-1);
	u.PushBack(2);
	Print(u);

	cout << "Checking..." << endl;

	Check(u);

	srand((unsigned int)time(NULL));
	for (int i = 0; i < 10000; i++)
		u.PushBack(rand());
	Check(u);

	cout << "Checks finished." << endl;
}
