// Copyright (C) 2005 Christian Mendl

#include "unique.h"
#include <time.h>
#include <iostream>
using namespace std;


template<class T>
void Print(const Unique<T>& u)
{
	cout << "Size: " << u.Size() << "; Number of bytes: " << u.NumBytes() << endl;
	if (!u.Empty())
	{
		cout << "Elements: ";
		for (int i = 0; i < u.Size()-1; i++)
			cout << u[i] << ", ";
		cout << u.Back() << endl;
	}
}


// keeps track of number of allocated objects
struct T
{
	T()
	{
		d = 1.0;

		++numAlloc;
		if (maxAlloc < numAlloc)
			maxAlloc = numAlloc;
	}

	T(double _d)
	{
		assert(_d != 0.0);

		d = _d;

		++numAlloc;
		if (maxAlloc < numAlloc)
			maxAlloc = numAlloc;
	}

	T(const T& that)
	{
		d = that.d;

		++numAlloc;
		if (maxAlloc < numAlloc)
			maxAlloc = numAlloc;
	}

	T& operator= (const T& that)
	{
		d = that.d;

		return *this;
	}

	~T()
	{
		assert(d != 0.0);

		--numAlloc;
		d = 0.0;
	}

	static int numAlloc, maxAlloc;

	double d;
};

int T::numAlloc = 0;
int T::maxAlloc = 0;


inline bool operator< (const T& left, const T& right)
{
	return left.d < right.d;
}


ostream& operator<< (ostream& strm, const T& t)
{
	strm << "<" << t.d << ">";
	return strm;
}


void main()
{
	cout << "Creating unique array and adding some elements..." << endl;
	Unique<T> u;
	u.PushBack(T(4.0));
	u.PushBack(T(2.0));
	u.PushBack(T(3.0));
	u.PushBack(T(-1.0));
	u.PushBack(T(2.0));
	Print(u);

	cout << endl << "Clearing..." << endl;
	u.Clear();

	cout << endl << "Number allocated: " << T::numAlloc << endl << "Maximum allocated: " << T::maxAlloc << endl;

	cout << endl << "Adding up to 10000 entries..." << endl;
	srand((unsigned int)time(NULL));
	for (int i = 0; i < 10000; i++)
	{
		double d = (double)rand()/RAND_MAX;
		if (d == 0.0)
			d = 1.0;

		u.PushBack(T(d));
	}

	cout << endl << "Erasing elements..." << endl;
	u.EraseAt(1);
	u.EraseAt(1);
	while (!u.Empty())
		u.Erase(u[0]);

	cout << endl << "Number allocated: " << T::numAlloc << endl << "Maximum allocated: " << T::maxAlloc << endl;
}
